<?php

namespace addons\kouming_domain;

use Think\Db;
use app\admin\lib\Plugin;

class KoumingDomainPlugin extends Plugin
{
    # 插件基本信息
    public $info = array(
        'name' => 'KoumingDomain', //插件英文名，改成你的插件英文就行了
        'title' => '口名网',
        'description' => '从这里开始，集成口名网，进行API 域名注册、续费、过户、解析等',
        'status' => 1,           //状态
        'author' => '口名',  //开发者
        'version' => '1.1',      // 版本号
        'module' => 'addons', //插件模块
        'lang' => [ # 一级菜单语言
            'chinese' => '口名网-域名', # 中文
            'chinese_tw' => '口名網-域名', # 台湾
            'english' => 'Kouming domain', # 英文
        ]
    );

    # 插件安装
    public function install()
    {
        $db_config = Db::getConfig();
        $kouming_domain_table = $this->tableConfig();
        $table_list_query = Db::query("SELECT table_name FROM information_schema.TABLES WHERE TABLE_SCHEMA='" . $db_config['database'] . "'");
        $table_name_array = array_column($table_list_query, "table_name");
        foreach ($kouming_domain_table as $k => $v) {
            $table_name = $db_config["prefix"] . $k;
            if (!in_array($table_name, $table_name_array)) {
                $table_content = str_replace("table_name", $table_name, $v);
                Db::query($table_content);
            }
        }
        return true;
    }

    # 插件卸载
    public function uninstall()
    {
        return true;
    }

    # 数据库表配置
    private function tableConfig()
    {
        return [
            "kouming_config" => "CREATE TABLE `table_name` (
  `name` varchar(255) NOT NULL COMMENT '配置名称',
  `config` text,
  PRIMARY KEY (`name`),
  KEY `user` (`name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC COMMENT='kouming配置';",
            "kouming_domain_log" => "CREATE TABLE `table_name` (
  `id` int(255) unsigned NOT NULL AUTO_INCREMENT,
  `uid` int(11) DEFAULT NULL COMMENT '所属用户',
  `aid` int(11) DEFAULT NULL COMMENT '所属管理员',
  `type` char(255) DEFAULT NULL COMMENT '类型',
  `remark` text COMMENT '简介',
  `date` datetime DEFAULT NULL COMMENT '时间',
  `ip` char(255) DEFAULT NULL COMMENT 'ip地址',
  `status` char(255) DEFAULT NULL COMMENT '状态',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC COMMENT='kouming日志';",
            "kouming_domain_list" => "CREATE TABLE `table_name` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `uid` int(11) DEFAULT NULL COMMENT '所属用户',
  `domain` varchar(255) DEFAULT NULL COMMENT '域名',
  `suffix` char(50) DEFAULT NULL COMMENT '域名后缀例如 .com',
  `language` enum('G','E') DEFAULT 'G' COMMENT '域名语言类型  E 表示英文、G 表示 中文',
  `template` int(11) DEFAULT NULL COMMENT '所属联系人模板',
  `dns` text COMMENT '域名解析DNS，多个,号分割',
  `start_date` datetime DEFAULT NULL COMMENT '注册时间',
  `end_date` datetime DEFAULT NULL COMMENT '过期时间',
  `contact_type` enum('1','2') DEFAULT '1' COMMENT '信息类型 1,个人，2企业',
  `contact_detail` text COMMENT '所有者信息',
  `status` enum('0','1','2','3') DEFAULT '0' COMMENT '状态（0正常，1禁用，2注册失败 , 3正在注册）',
  `remark` text COMMENT '备注',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `domain` (`domain`) USING BTREE,
  KEY `user` (`uid`) USING BTREE,
  KEY `suffix` (`suffix`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC COMMENT='kouming域名';",
            "kouming_domain_parse" => "CREATE TABLE `table_name` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `host_id` char(50) DEFAULT NULL COMMENT '外部DNS记录编号',
  `domain` char(255) DEFAULT NULL COMMENT '域名',
  `host` varchar(255) DEFAULT NULL COMMENT '主机记录',
  `type` varchar(255) DEFAULT NULL COMMENT '解析类型',
  `value` text COMMENT '记录值',
  `ttl` varchar(255) DEFAULT NULL COMMENT 'ttl',
  `mx` varchar(255) DEFAULT NULL COMMENT 'mx优先级',
  PRIMARY KEY (`id`),
  KEY `domain` (`domain`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC COMMENT='kouming域名解析'",
            "kouming_domain_suffix" => "CREATE TABLE `table_name` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `suffix` char(50) DEFAULT NULL COMMENT '域名后缀例如 .com',
  `language` enum('G','E') DEFAULT 'G' COMMENT 'E 表示英文、G 表示 中文',
  `status` enum('0','1') DEFAULT '0' COMMENT '状态（0正常，1禁用）',
  `create` decimal(10,2) DEFAULT '0.00' COMMENT '注册价格',
  `renewal` decimal(10,2) DEFAULT '0.00' COMMENT '续费价格',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC COMMENT='kouming域名后缀';",
            "kouming_domain_template" => "CREATE TABLE `table_name` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `category` varchar(255) DEFAULT NULL COMMENT '模板类别例如 : 0通用模板',
  `uid` int(11) DEFAULT NULL COMMENT '所属用户',
  `default` tinyint(1) DEFAULT '0' COMMENT '是否默认模板（0否，1是）',
  `type` tinyint(4) DEFAULT NULL COMMENT '模板类型 1,个人，2企业',
  `detail` text COMMENT '所有者信息',
  `date` datetime DEFAULT NULL COMMENT '添加时间',
  `external` varchar(255) DEFAULT NULL COMMENT '外部模板编号',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC COMMENT='kouming联系人模版';"
        ];
    }
}