<?php

namespace addons\kouming_domain\controller\clientarea;

use addons\kouming_domain\lib\Helper;
use think\Db;

class DomainController extends BaseController
{
    private function update_domain_status()
    {
        $domain = Db::name('kouming_domain_list')->where([
            'status' => "3",
            'uid' => $this->uid,
        ])->select()->toArray();
        //接口
        $url_path = "/domainManagement/detail";
        $public_parameter = Helper::publicParameter($this->config["user_id"], $this->config["api_key"]);
        foreach ($domain as $k => $v) {
            $status = $v["status"];
            $api_data = [
                "domainName" => $v["domain"]
            ];
            $api_result = Helper::objectArray(json_decode(Helper::post($this->config["api_url"] . $url_path, array_merge($public_parameter, $api_data))));
            //Helper::writeApiLog($url_path, $api_data, $api_result);
            if (!$api_result["code"]) {
                if ($api_result["data"]["status"]) {
                    switch ($api_result["data"]["status"]) {
                        case "1":
                            $status = "0";
                            break;
                        case "2":
                            $status = "2";
                            break;
                        case "0":
                            $status = "3";
                            break;
                    }
                }
            } else {
                $status = "2";
            }
            if ($status != "3") {
                Db::name("kouming_domain_list")->where("id", $v["id"])->update([
                    "status" => $status,
                    "dns" => $api_result["data"]["dns"],
                    "remark" => $api_result["data"]["statusRemark"]
                ]);
            }
        }
    }

    private function get_suffix()
    {
        $domain_suffix = Db::name("kouming_domain_suffix")->where("status", "0")->select()->toArray();
        $domain_suffix_new = [];
        foreach ($domain_suffix as $k => $v) {
            unset($v["id"]);
            unset($v["status"]);
            $domain_suffix_new[$k] = $v;
        }
        return $domain_suffix_new;
    }

    private function get_template()
    {
        $domain_template = Db::name("kouming_domain_template")->where("uid", $this->uid)->select()->toArray();
        $domain_template_new = [];
        foreach ($domain_template as $k => $v) {
            unset($v["external"]);
            unset($v["date"]);
            unset($v["uid"]);
            $v["detail"] = Helper::objectArray(json_decode($v["detail"]));
            $domain_template_new[$k] = $v;
        }
        return $domain_template_new;
    }

    private function get_domain_type($domain)
    {
        $language = "E";
        if (mb_strlen($domain, "utf-8") != strlen($domain)) {
            $language = "G";
        }
        $domain_array = explode(".", $domain, 2);
        return [
            "domain" => $domain,
            "suffix" => "." . $domain_array[1],
            "language" => $language
        ];
    }

    # 我的域名
    public function index()
    {
        $this->update_domain_status();
        //列表
        if ($this->request->param('method') == "list") {
            //按字段排序默认ID字段
            $field = $this->request->param('field');
            //升序降序默认desc
            $sort = $this->request->param('sort');
            //默认排序
            if (!$field or !$sort) {
                $field = "id";
                $sort = "desc";
            }
            //每页显示默认10条
            $perpage = $this->request->param('limit') ?? 10;
            //页码无数据1
            $page = $this->request->param('page') ?? 1;
            //查询条件
            $where = [];
            //状态
            if ($this->request->param('status') != "") {
                $v = $this->request->param('status');
                if ($v == "expired") {
                    $where[] = [
                        "end_date",
                        "<=",
                        date("Y-m-d")
                    ];
                } else {
                    $where[] = [
                        "status",
                        "=",
                        $v
                    ];
                }
            }
            //域名
            if ($this->request->param('domain') != "") {
                $v = $this->request->param('domain');
                $where[] = [
                    "domain",
                    "=",
                    $v
                ];
            }
            //后缀
            if ($this->request->param('suffix') != "") {
                $v = $this->request->param('suffix');
                $where[] = [
                    "suffix",
                    "=",
                    $v
                ];
            }
            //类型语言
            if ($this->request->param('language') != "") {
                $v = $this->request->param('language') == "E" ? "E" : "G";
                $where[] = [
                    "language",
                    "=",
                    $v
                ];
            }
            $where[] = ["uid", "=", $this->uid];
            //取得总数
            $total = Db::name('kouming_domain_list')->where($where)->order($field, $sort)->count();
            //判断页码
            $true_page = ceil($total / $perpage);
            if ($page > $true_page) {
                $page = 1;
            }
            //如果显示条数大于总数分页关闭
            if ($perpage > $total) {
                $page = 1;
            }
            //分页
            $start = max(0, ($page - 1) * $perpage);
            //查数据
            $data = Db::name('kouming_domain_list')->where($where)->order($field, $sort)->limit($start, $perpage)->select()->toArray();
            //重组数据
            $dataS = [];
            foreach ($data as $k => $v) {
                unset($data[$k]);
                if ($v["end_date"] <= date("Y-m-d")) {
                    $v["status"] = "expired";
                }
                unset($v["uid"]);
                $dataS[$k] = $v;
            }
            $data = array(
                "count" => $total,
                "list" => $dataS
            );
            return Helper::message(0, '查询成功', '', false, '', $data);
        }
        $this->assign('Title', "我的域名");
        return $this->fetch('/domain_index');
    }

    # 查询域名
    public function inquire()
    {
        //查询
        if ($this->request->param('method') == "action") {
            $domain = $this->request->param('domain');
            if (!$domain) {
                return Helper::message(1, '请输入域名');
            }
            $domain_type = $this->get_domain_type($domain);
            $domain_suffix = Db::name("kouming_domain_suffix")->where([
                'suffix' => $domain_type["suffix"],
                'language' => $domain_type["language"],
                'status' => "0",
            ])->find();
            if (!$domain_suffix["id"]) {
                return Helper::message(1, '域名后缀不支持');
            }
            //接口查询
            $url_path = "/domainBasis/search";
            $public_parameter = Helper::publicParameter($this->config["user_id"], $this->config["api_key"]);
            $api_data = [
                "domainName" => $domain
            ];
            $api_result = Helper::objectArray(json_decode(Helper::post($this->config["api_url"] . $url_path, array_merge($public_parameter, $api_data))));
            Helper::writeApiLog($url_path, $api_data, $api_result);
            $data = [
                "domain" => $domain,
                "language" => $domain_suffix["language"],
                "suffix" => $domain_suffix["suffix"],
                "status" => strval($api_result["code"]),
                "price" => $domain_suffix["create"],
            ];
            return Helper::message(0, '', '', false, '', $data);

        }
        $this->assign('domain_suffix', $this->get_suffix());
        $this->assign('domain_template', $this->get_template());
        $this->assign('Title', "注册域名");
        return $this->fetch('/domain_inquire');
    }

    # 注册域名
    public function register()
    {
        //列表
        if ($this->request->param('method') == "action") {
            $domain = $this->request->param('domain');
            $year = intval($this->request->param('year'));
            $template = intval($this->request->param('template'));
            if (!$domain) {
                return Helper::message(1, '域名不能位空');
            }
            $domain_type = $this->get_domain_type($domain);
            $domain_suffix = Db::name("kouming_domain_suffix")->where([
                'suffix' => $domain_type["suffix"],
                'language' => $domain_type["language"],
                'status' => "0",
            ])->find();
            if (!$domain_suffix["id"]) {
                return Helper::message(1, '域名后缀不支持，请联系管理员');
            }
            if (!$year or $year > 10) {
                return Helper::message(1, '注册年限错误');
            }
            if (!$template) {
                return Helper::message(1, '联系人模板为空');
            }
            $domain_template = Db::name("kouming_domain_template")->where([
                'id' => $template,
                'uid' => $this->uid
            ])->find();
            if (!$domain_template["id"]) {
                return Helper::message(1, '联系人模板非法');
            }
            //域名是否存在
            $is_exist_domain = Db::name("kouming_domain_list")->where("domain", $domain)->find();
            if ($is_exist_domain["id"]) {
                return Helper::message(1, '域名已存在');
            }
           //计算价格
            $domain_price = number_format($domain_suffix["create"], 2, ".", false);
            if ($year > 1) {
                $domain_renewal_year = $year - 1;
                $domain_price = number_format(($domain_suffix["renewal"] * $domain_renewal_year) + $domain_suffix["create"], 2, ".", false);
            }
            $user_money = Db::name("Clients")->where("id", $this->uid)->value("credit");
            $balance = $user_money - $domain_price;
            if ($balance < 0) {
                return Helper::message(1, '余额不足');
            }
            //扣款
            Db::startTrans();
            try {
                $start_date = date("Y-m-d H:i:s");
                $end_date = date("Y-m-d H:i:s", strtotime("+" . $year . " year"));
                $invoices_id = Db::name("invoices")->insertGetId([
                    'uid' => $this->uid,
                    'invoice_num' => date("YmdHis") . mt_rand(10000, 999999),
                    'create_time' => strtotime($start_date),
                    'subtotal' => $domain_price,
                    'status' => 'Paid',
                    'paid_time' => strtotime($start_date),
                    'credit' => $domain_price,
                    'payment' => 'UserCustom',
                    'type' => 'product',
                ]);
                $invoices_items_id = Db::name("invoice_items")->insertGetId([
                    'uid' => $this->uid,
                    'invoice_id' => $invoices_id,
                    'description' => "注册域名" . $domain . ", 年限" . $year . "年",
                    'rel_id' => 0,
                    'amount' => $domain_price,
                    'payment' => 'UserCustom',
                    'type' => 'domain',
                ]);
                $credit_id = Db::name("credit")->insertGetId([
                    'uid' => $this->uid,
                    'create_time' => strtotime($start_date),
                    'description' => "Credit Applied to Invoice #" . $invoices_id,
                    'relid' => $invoices_id,
                    'balance' => $balance,
                    'amount' => $domain_price
                ]);
                Db::name("Clients")->where("id", $this->uid)->setDec("credit", $domain_price);
                //接口
                $url_path = "/domainBasis/create";
                $public_parameter = Helper::publicParameter($this->config["user_id"], $this->config["api_key"]);
                $api_data = [
                    "createData" => json_encode([[
                        "domainName" => $domain,
                        "templateId" => $domain_template["external"],
                        "years" => $year
                    ]])
                ];
                $api_result = Helper::objectArray(json_decode(Helper::post($this->config["api_url"] . $url_path, array_merge($public_parameter, $api_data))));
                Helper::writeApiLog($url_path, $api_data, $api_result);
                if ($api_result["code"] or !isset($api_result["data"]["failure"]) or $api_result["data"]["failure"] > 0) {
                    throw new \Exception("注册失败");
                }
                //域名入库
                $add_data = [
                    "uid" => $this->uid,
                    "domain" => $domain,
                    "suffix" => $domain_suffix["suffix"],
                    "language" => $domain_suffix["language"],
                    "template" => $domain_template["id"],
                    "dns" => "",
                    "start_date" => $start_date,
                    "end_date" => $end_date,
                    "contact_type" => $domain_template["type"],
                    "contact_detail" => $domain_template["detail"],
                    "status" => "3",
                    "remark" => ""
                ];
                Db::name("kouming_domain_list")->insert($add_data);
                Db::commit();
            } catch (\Exception $e) {
                Db::rollback();
                return Helper::message(1, '注册失败，请联系管理员');
            }
            //注册成功
            return Helper::message(0, "注册成功", 'href', true, "注册域名成功：" . json_encode($add_data), ["url" => shd_addon_url('KoumingDomain://Domain/index', [], true)]);
        }
    }

    # 联系人模板
    public function template()
    {
        //列表
        if ($this->request->param('method') == "list") {
            //按字段排序默认ID字段
            $field = $this->request->param('field');
            //升序降序默认desc
            $sort = $this->request->param('sort');
            //默认排序
            if (!$field or !$sort) {
                $field = "id";
                $sort = "desc";
            }
            //每页显示默认10条
            $perpage = $this->request->param('limit') ?? 10;
            //页码无数据1
            $page = $this->request->param('page') ?? 1;
            //查询条件
            $where = [];
            if ($this->request->param('remark') and $this->request->param('remark') != "") {
                $v = $this->request->param('remark');
                $where[] = [
                    "detail",
                    "like",
                    "%$v%"
                ];
            }
            $where[] = ["uid", "=", $this->uid];
            //取得总数
            $total = Db::name('kouming_domain_template')->where($where)->order($field, $sort)->count();
            //判断页码
            $true_page = ceil($total / $perpage);
            if ($page > $true_page) {
                $page = 1;
            }
            //如果显示条数大于总数分页关闭
            if ($perpage > $total) {
                $page = 1;
            }
            //分页
            $start = max(0, ($page - 1) * $perpage);
            //查数据
            $data = Db::name('kouming_domain_template')->where($where)->order($field, $sort)->limit($start, $perpage)->select()->toArray();
            //重组数据
            $dataS = [];
            foreach ($data as $k => $v) {
                unset($data[$k]);
                unset($v["uid"]);
                unset($v["external"]);
                $dataS[$k] = $v;
            }
            $data = array(
                "count" => $total,
                "list" => $dataS
            );
            return Helper::message(0, '查询成功', '', false, '', $data);
        }
        //设置默认模板
        if ($this->request->param('method') == "default") {
            $result = Db::name('kouming_domain_template')->where([
                'id' => intval($this->request->param('id')),
                'uid' => $this->uid
            ])->find();
            if (!$result["id"]) {
                return Helper::message(1, "该模板不存在");
            }
            //事务
            Db::startTrans();
            try {
                Db::name('kouming_domain_template')->where([
                    'uid' => $this->uid
                ])->update(["default" => 0]);
                Db::name('kouming_domain_template')->where([
                    'id' => $result["id"]
                ])->update(["default" => 1]);
                // 提交事务
                Db::commit();
            } catch (\Exception $e) {
                // 回滚事务
                Db::rollback();
            }
            return Helper::message(0, "设置成功", '', true, "设置默认联系人模板：" . $result["id"]);
        }
        //删除
        if ($this->request->param('method') == "delete") {
            $result = Db::name('kouming_domain_template')->where([
                'id' => intval($this->request->param('id')),
                'uid' => $this->uid
            ])->find();
            if (!$result["id"]) {
                return Helper::message(1, "该模板不存在");
            }
            //是否存在该业务
            $domain = Db::name('kouming_domain_list')->where('template', $result["id"])->find();
            if ($domain["id"]) {
                return Helper::message(1, "存在域名使用禁止删除");
            }
            //请求接口
            $url_path = "/domainTemplate/delete";
            $public_parameter = Helper::publicParameter($this->config["user_id"], $this->config["api_key"]);
            $api_data = [
                "templateId" => $result["external"]
            ];
            $api_result = Helper::objectArray(json_decode(Helper::post($this->config["api_url"] . $url_path, array_merge($public_parameter, $api_data))));
            Helper::writeApiLog($url_path, $api_data, $api_result);
            if ($api_result["code"]) {
                return Helper::message(1, "删除失败，请联系管理员");
            }
            Db::name('kouming_domain_template')->delete($result["id"]);
            return Helper::message(0, "删除成功", '', true, "删除联系人模板：" . json_encode($result));
        }
        //创建
        if ($this->request->param('method') == "create") {
            $detail_data = [
                "organization" => $this->request->param('organization'),
                "last_name" => $this->request->param('last_name'),
                "first_name" => $this->request->param('first_name'),
                "country_code" => $this->request->param('country_code'),
                "country" => $this->request->param('country'),
                "province" => $this->request->param('province'),
                "city" => $this->request->param('city'),
                "address" => $this->request->param('address'),
                "phone" => $this->request->param('phone'),
                "fax" => $this->request->param('fax') != "" ? $this->request->param('fax') : $this->request->param('phone'),
                "zip_code" => $this->request->param('zip_code'),
                "email" => $this->request->param('email'),
            ];
            //请求接口
            $url_path = "/domainTemplate/create";
            $public_parameter = Helper::publicParameter($this->config["user_id"], $this->config["api_key"]);
            $api_data = [
                "category" => "0",
                "enOrganization" => $detail_data["organization"],
                "enLastName" => $detail_data["last_name"],
                "enFirstName" => $detail_data["first_name"],
                "countryCode" => $detail_data["country_code"],
                "enCountries" => $detail_data["country"],
                "enProvince" => $detail_data["province"],
                "enCity" => $detail_data["city"],
                "enAddress" => $detail_data["address"],
                "phone" => $detail_data["phone"],
                "fax" => $detail_data["fax"],
                "zipCode" => $detail_data["zip_code"],
                "email" => $detail_data["email"],
            ];
            $api_result = Helper::objectArray(json_decode(Helper::post($this->config["api_url"] . $url_path, array_merge($public_parameter, $api_data))));
            Helper::writeApiLog($url_path, $api_data, $api_result);
            if ($api_result["code"] or !isset($api_result["data"]["templateId"])) {
                return Helper::message(1, "创建失败，请联系管理员");
            }
            $add_data = [
                'uid' => $this->uid,
                'default' => 0,
                'type' => $this->request->param('organization') ? 2 : 1,
                'detail' => json_encode($detail_data),
                'date' => date("Y-m-d H:i:s"),
                'external' => $api_result["data"]["templateId"]
            ];
            Db::name('kouming_domain_template')->insert($add_data);
            return Helper::message(0, "创建成功", '', true, "创建联系人模板：" . json_encode($add_data));
        }
        $this->assign('Title', "联系人模板");
        return $this->fetch('/domain_template');
    }

    # 域名续费
    public function renew()
    {
        if ($this->request->param('method') == "get_price") {
            return Helper::message(0, "", '', false, "", $this->get_suffix());
        }
        if ($this->request->param('method') == "action") {
            $domain = $this->request->param('domain');
            $year = intval($this->request->param('year'));
            if (!$year or $year > 10) {
                return Helper::message(1, '续费年限错误');
            }
            if (!$domain) {
                return Helper::message(1, '域名不能为空');
            }
            $domain_type = $this->get_domain_type($domain);
            //域名是否存在
            $is_exist_domain = Db::name("kouming_domain_list")->where([
                'domain' => $domain,
                'uid' => $this->uid
            ])->find();
            if (!$is_exist_domain["id"]) {
                return Helper::message(1, '域名不存在');
            }
            if ($is_exist_domain["status"] != "0" and $is_exist_domain["status"] != "1") {
                return Helper::message(1, '当前域名状态不允许续费');
            }
            $domain_suffix = Db::name("kouming_domain_suffix")->where([
                'suffix' => $domain_type["suffix"],
                'language' => $domain_type["language"]
            ])->find();
            if (!$domain_suffix["id"]) {
                return Helper::message(1, '域名后缀不支持，请联系管理员');
            }
            //计算价格
            $domain_price = number_format($domain_suffix["renewal"] * $year, 2, ".", false);
            $user_money = Db::name("Clients")->where("id", $this->uid)->value("credit");
            $balance = $user_money - $domain_price;
            if ($balance < 0) {
                return Helper::message(1, '余额不足');
            }
            //扣款
            Db::startTrans();
            try {
                $start_date = date("Y-m-d H:i:s");
                $invoices_id = Db::name("invoices")->insertGetId([
                    'uid' => $this->uid,
                    'invoice_num' => date("YmdHis") . mt_rand(10000, 999999),
                    'create_time' => strtotime($start_date),
                    'subtotal' => $domain_price,
                    'status' => 'Paid',
                    'paid_time' => strtotime($start_date),
                    'credit' => $domain_price,
                    'payment' => 'UserCustom',
                    'type' => 'product',
                ]);
                $invoices_items_id = Db::name("invoice_items")->insertGetId([
                    'uid' => $this->uid,
                    'invoice_id' => $invoices_id,
                    'description' => "续费域名" . $domain . ", 年限" . $year . "年",
                    'rel_id' => 0,
                    'amount' => $domain_price,
                    'payment' => 'UserCustom',
                    'type' => 'domain',
                ]);
                $credit_id = Db::name("credit")->insertGetId([
                    'uid' => $this->uid,
                    'create_time' => strtotime($start_date),
                    'description' => "Credit Applied to Invoice #" . $invoices_id,
                    'relid' => $invoices_id,
                    'balance' => $balance,
                    'amount' => $domain_price
                ]);
                Db::name("Clients")->where("id", $this->uid)->setDec("credit", $domain_price);
                //接口
                $url_path = "/domainManagement/renewal";
                $public_parameter = Helper::publicParameter($this->config["user_id"], $this->config["api_key"]);
                $api_data = [
                    "renewalData" => json_encode([[
                        "domainName" => $domain,
                        "years" => $year
                    ]])
                ];
                $api_result = Helper::objectArray(json_decode(Helper::post($this->config["api_url"] . $url_path, array_merge($public_parameter, $api_data))));
                Helper::writeApiLog($url_path, $api_data, $api_result);
                if ($api_result["code"] or !isset($api_result["data"]["failure"]) or $api_result["data"]["failure"] > 0) {
                    throw new \Exception("续费失败");
                }
                //修改域名年限
                $update_data = [
                    "end_date" => date("Y-m-d H:i:s", strtotime("+" . $year . " year", strtotime($is_exist_domain["end_date"]))),
                ];
                Db::name("kouming_domain_list")->where("id", $is_exist_domain["id"])->update($update_data);
                Db::commit();
            } catch (\Exception $e) {
                Db::rollback();
                return Helper::message(1, '续费失败，请联系管理员');
            }
            //续费成功
            $new_domain = Db::name("kouming_domain_list")->where("id", $is_exist_domain["id"])->find();
            return Helper::message(0, "续费成功", 'reload', true, "续费域名成功：" . json_encode($is_exist_domain) . "-to-" . json_encode($new_domain));
        }
    }

    # 域名过户
    public function transfer()
    {
        if ($this->request->param('method') == "get_template") {
            return Helper::message(0, "", '', false, "", $this->get_template());
        }
        if ($this->request->param('method') == "action") {
            $domain = $this->request->param('domain');
            $template = intval($this->request->param('template'));
            if (!$domain) {
                return Helper::message(1, '域名不能为空');
            }
            if (!$template) {
                return Helper::message(1, '联系人模板不能为空');
            }
            //域名是否存在
            $is_exist_domain = Db::name("kouming_domain_list")->where([
                'domain' => $domain,
                'uid' => $this->uid
            ])->find();
            if (!$is_exist_domain["id"]) {
                return Helper::message(1, '域名不存在');
            }
            if ($is_exist_domain["status"] != "0") {
                return Helper::message(1, '当前域名状态不允许过户');
            }
            if ($is_exist_domain["end_date"] <= date("Y-m-d")) {
                return Helper::message(1, '过期域名不允许过户');
            }
            if ($template == $is_exist_domain["template"]) {
                return Helper::message(1, '不能过户到相同联系人模板下');
            }
            $domain_template = Db::name("kouming_domain_template")->where([
                'id' => $template,
                'uid' => $this->uid
            ])->find();
            if (!$domain_template["id"]) {
                return Helper::message(1, '联系人模板非法');
            }
            //接口
            $url_path = "/domainManagement/transfer";
            $public_parameter = Helper::publicParameter($this->config["user_id"], $this->config["api_key"]);
            $api_data = [
                "transferData" => json_encode([[
                    "domainName" => $domain,
                    "templateId" => $domain_template["external"]
                ]])
            ];
            $api_result = Helper::objectArray(json_decode(Helper::post($this->config["api_url"] . $url_path, array_merge($public_parameter, $api_data))));
            Helper::writeApiLog($url_path, $api_data, $api_result);
            if ($api_result["code"] or !isset($api_result["data"]["failure"]) or $api_result["data"]["failure"] > 0) {
                if ($api_result["data"]["failureMessage"][$domain] == "domain name Transfer in progress") {
                    return Helper::message(1, '请等待上一次过户完成');
                }
                return Helper::message(1, '过户失败，请联系管理员');
            }
            //修改域名模板
            $update_data = [
                "template" => $domain_template["id"],
                "contact_type" => $domain_template["type"],
                "contact_detail" => $domain_template["detail"]
            ];
            Db::name("kouming_domain_list")->where("id", $is_exist_domain["id"])->update($update_data);
            //过户成功
            $new_domain = Db::name("kouming_domain_list")->where("id", $is_exist_domain["id"])->find();
            return Helper::message(0, "过户成功", 'reload', true, "过户域名成功：" . json_encode($is_exist_domain) . "-to-" . json_encode($new_domain));
        }
    }

    # 域名DNS
    public function dns()
    {
        if ($this->request->param('method') == "action") {
            $domain = $this->request->param('domain');
            $dns_type = intval($this->request->param('dns_type'));
            $dns = Helper::toSemiangle($this->request->param('dns'));
            if (!$domain) {
                return Helper::message(1, '域名不能为空');
            }
            if ($dns_type == "2") {
                if (!$dns) {
                    return Helper::message(1, 'DNS地址不能为空');
                }
                $dns_array = explode(",", $dns);
                if (count($dns_array) < 2) {
                    return Helper::message(1, 'DNS地址至少2条');
                }
            }
            //域名是否存在
            $is_exist_domain = Db::name("kouming_domain_list")->where([
                'domain' => $domain,
                'uid' => $this->uid
            ])->find();
            if (!$is_exist_domain["id"]) {
                return Helper::message(1, '域名不存在');
            }
            if ($is_exist_domain["status"] != "0") {
                return Helper::message(1, '当前域名状态不允许修改DNS');
            }
            if ($is_exist_domain["end_date"] <= date("Y-m-d")) {
                return Helper::message(1, '过期域名不允许修改DNS');
            }
            //接口
            $url_path = "/domainManagement/dns";
            $public_parameter = Helper::publicParameter($this->config["user_id"], $this->config["api_key"]);
            $api_data = [
                "domainName" => $domain,
                "dnsType" => $dns_type == "2" ? 0 : 1,
                "dns" => $dns,
            ];
            $api_result = Helper::objectArray(json_decode(Helper::post($this->config["api_url"] . $url_path, array_merge($public_parameter, $api_data))));
            Helper::writeApiLog($url_path, $api_data, $api_result);
            if ($api_result["code"]) {
                return Helper::message(1, 'DNS修改失败，请联系管理员');
            }
            //修改域名DNS
            $update_data = [
                "dns" => $api_result["data"]["dns"]
            ];
            Db::name("kouming_domain_list")->where("id", $is_exist_domain["id"])->update($update_data);
            //修改DNS成功
            $new_domain = Db::name("kouming_domain_list")->where("id", $is_exist_domain["id"])->find();
            return Helper::message(0, "修改DNS成功", 'reload', true, "域名修改DNS成功：" . json_encode($is_exist_domain) . "-to-" . json_encode($new_domain));
        }
    }

    //验证解析结果
    private function parse_verify($message)
    {
        switch ($message) {
            case "domainName cannot be empty":
                return "域名不能为空";
                break;
            case "domain name error or not support":
                return "域名错误或不支持";
                break;
            case "Domain does not belong to you or is not registered":
                return "域名不属于你或未注册";
                break;
            case "Domain name Expired":
                return "域名过期";
                break;
            case "The current domain name status does not allow":
                return "当前状态不允许";
                break;
            case "domain name temporary template does not allow operation":
                return "临时模板不允许";
                break;
            case "Custom DNS":
                return "自定义DNS不能本站解析";
                break;
            case "host cannot be empty":
                return "host不能为空";
                break;
            case "type cannot be empty":
                return "type不能为空";
                break;
            case "type error":
                return "type错误";
                break;
            case "value cannot be empty":
                return "value不能为空";
                break;
            case "mx cannot be empty":
                return "mx不能为空";
                break;
            case "recordId cannot be empty":
                return "recordId不能为空";
                break;
            case "record does not exist":
                return "记录不存在";
                break;
            case "failure operation":
                return "操作失败";
                break;
        }
    }

    # 解析域名
    public function parse()
    {
        $id = intval($this->request->param('id'));
        $domain = $this->request->param('domain');
        if (!$domain) {
            return Helper::message(1, '域名不能为空');
        }
        //域名是否存在
        $is_exist_domain = Db::name("kouming_domain_list")->where([
            'domain' => $domain,
            'uid' => $this->uid
        ])->find();
        if (!$is_exist_domain["id"]) {
            return Helper::message(1, '域名不存在');
        }
        if ($is_exist_domain["status"] != '0') {
            return Helper::message(1, '当前域名状态不允操作解析');
        }
        if ($is_exist_domain["end_date"] <= date("Y-m-d")) {
            return Helper::message(1, '过期域名不允操作解析');
        }
        $parse_data = [
            "domain" => $domain,
            "host" => $this->request->param('host'),
            "type" => $this->request->param('type'),
            "value" => $this->request->param('value'),
            "ttl" => $this->request->param('ttl'),
            "mx" => $this->request->param('mx')
        ];
        //同步
        if ($this->request->param('method') == "synchronization") {
            //接口
            $url_path = "/domainManagement/parsing";
            $public_parameter = Helper::publicParameter($this->config["user_id"], $this->config["api_key"]);
            $api_data = [
                "domainName" => $domain,
                "methods" => "list"
            ];
            $api_result = Helper::objectArray(json_decode(Helper::post($this->config["api_url"] . $url_path, array_merge($public_parameter, $api_data))));
            //Helper::writeApiLog($url_path, $api_data, $api_result);
            if ($api_result["code"]) {
                return Helper::message(1, '同步解析失败（' . $this->parse_verify($api_result["message"]) . '），请联系管理员');
            }
            //删除所有解析重新添加
            Db::name('kouming_domain_parse')->where("domain", $domain)->delete();
            foreach ($api_result["data"] as $k => $v) {
                Db::name("kouming_domain_parse")->insert([
                    "domain" => $domain,
                    "host_id" => $v["id"],
                    "host" => $v["host"],
                    "type" => $v["type"],
                    "value" => $v["value"],
                    "ttl" => $v["ttl"],
                    "mx" => $v["mx"]
                ]);
            }
            //同步解析成功
            return Helper::message(0, "同步解析成功");
        }
        //列表
        if ($this->request->param('method') == "list") {
            //按字段排序默认ID字段
            $field = $this->request->param('field');
            //升序降序默认desc
            $sort = $this->request->param('sort');
            //默认排序
            if (!$field or !$sort) {
                $field = "id";
                $sort = "desc";
            }
            //每页显示默认10条
            $perpage = $this->request->param('limit') ?? 10;
            //页码无数据1
            $page = $this->request->param('page') ?? 1;
            //查询条件
            $where = [];
            //域名
            $where[] = [
                "domain",
                "=",
                $domain
            ];
            //解析类型
            if ($this->request->param('type') != "") {
                $v = $this->request->param('type');
                $where[] = [
                    "type",
                    "=",
                    $v
                ];
            }
            //主机记录
            if ($this->request->param('host') != "") {
                $v = $this->request->param('host');
                $where[] = [
                    "host",
                    "=",
                    $v
                ];
            }
            //记录值
            if ($this->request->param('value') != "") {
                $v = $this->request->param('value');
                $where[] = [
                    "value",
                    "=",
                    $v
                ];
            }
            //取得总数
            $total = Db::name('kouming_domain_parse')->where($where)->order($field, $sort)->count();
            //判断页码
            $true_page = ceil($total / $perpage);
            if ($page > $true_page) {
                $page = 1;
            }
            //如果显示条数大于总数分页关闭
            if ($perpage > $total) {
                $page = 1;
            }
            //分页
            $start = max(0, ($page - 1) * $perpage);
            //查数据
            $data = Db::name('kouming_domain_parse')->where($where)->order($field, $sort)->limit($start, $perpage)->select()->toArray();
            //重组数据
            $dataS = [];
            foreach ($data as $k => $v) {
                unset($data[$k]);
                unset($v["host_id"]);
                unset($v["line"]);
                $dataS[$k] = $v;
            }
            $data = array(
                "count" => $total,
                "list" => $dataS
            );
            return Helper::message(0, '查询成功', '', false, '', $data);
        }
        //添加
        if ($this->request->param('method') == "add") {
            //接口
            $url_path = "/domainManagement/parsing";
            $public_parameter = Helper::publicParameter($this->config["user_id"], $this->config["api_key"]);
            $api_data = [
                "domainName" => $domain,
                "methods" => "add",
                "host" => $parse_data["host"],
                "type" => $parse_data["type"],
                "value" => $parse_data["value"],
                "ttl" => $parse_data["ttl"],
                "mx" => $parse_data["mx"]
            ];
            $api_result = Helper::objectArray(json_decode(Helper::post($this->config["api_url"] . $url_path, array_merge($public_parameter, $api_data))));
            Helper::writeApiLog($url_path, $api_data, $api_result);
            if ($api_result["code"]) {
                return Helper::message(1, '添加解析失败（' . $this->parse_verify($api_result["message"]) . '），请联系管理员');
            }
            $parse_data["host_id"] = $api_result["data"]["recordId"];
            Db::name("kouming_domain_parse")->insert($parse_data);
            //添加解析成功
            return Helper::message(0, "添加解析成功", '', true, "添加域名解析：" . $domain . "-to-" . json_encode($parse_data));
        }
        //编辑
        if ($this->request->param('method') == "edit") {
            $parse_record = Db::name("kouming_domain_parse")->where([
                'domain' => $domain,
                'id' => $id
            ])->find();
            if (!$parse_record["id"]) {
                return Helper::message(1, '解析不存在');
            }
            //接口
            $url_path = "/domainManagement/parsing";
            $public_parameter = Helper::publicParameter($this->config["user_id"], $this->config["api_key"]);
            $api_data = [
                "domainName" => $domain,
                "methods" => "edit",
                "recordId" => $parse_record["host_id"],
                "host" => $parse_data["host"],
                "type" => $parse_data["type"],
                "value" => $parse_data["value"],
                "ttl" => $parse_data["ttl"],
                "mx" => $parse_data["mx"]
            ];
            $api_result = Helper::objectArray(json_decode(Helper::post($this->config["api_url"] . $url_path, array_merge($public_parameter, $api_data))));
            Helper::writeApiLog($url_path, $api_data, $api_result);
            if ($api_result["code"]) {
                return Helper::message(1, '编辑解析失败（' . $this->parse_verify($api_result["message"]) . '），请联系管理员');
            }
            Db::name("kouming_domain_parse")->where("id", $parse_record["id"])->update($parse_data);
            //编辑解析成功
            $new_parse_data = Db::name("kouming_domain_parse")->where("id", $parse_record["id"])->find();
            return Helper::message(0, "编辑解析成功", '', true, "编辑域名解析：" . $domain . "-to-" . json_encode($parse_data) . "-to-" . json_encode($new_parse_data));
        }
        //删除
        if ($this->request->param('method') == "delete") {
            $parse_record = Db::name("kouming_domain_parse")->where([
                'domain' => $domain,
                'id' => $id
            ])->find();
            if (!$parse_record["id"]) {
                return Helper::message(1, '解析不存在');
            }
            //接口
            $url_path = "/domainManagement/parsing";
            $public_parameter = Helper::publicParameter($this->config["user_id"], $this->config["api_key"]);
            $api_data = [
                "domainName" => $domain,
                "methods" => "delete",
                "recordId" => $parse_record["host_id"]
            ];
            $api_result = Helper::objectArray(json_decode(Helper::post($this->config["api_url"] . $url_path, array_merge($public_parameter, $api_data))));
            Helper::writeApiLog($url_path, $api_data, $api_result);
            if ($api_result["code"]) {
                return Helper::message(1, '删除解析失败（' . $this->parse_verify($api_result["message"]) . '），请联系管理员');
            }
            Db::name('kouming_domain_parse')->delete($parse_record["id"]);
            return Helper::message(0, "删除解析成功", '', true, "删除域名解析：" . $domain . "-to-" . json_encode($parse_record));
        }
    }
}